<?php

/**
 * Plugin Name: CasinoWare
 * Author: Jens Harttmann
 * Author URI: https://casinoware.com
 */

defined('ABSPATH') or die('No direct access');

class CWPlugIn
{
	const VERSION = '0.9.14';
	const UPDATE_URI = "https://casinoware.com/downloads/wp/casinoware-plugin.php";

	// Admin error messages from update
	protected $errorMsgs = null;
	// Id for several lists on the same page
	protected $listID = 1;

	/* ---------------------------------------------- */
	/* Activation and deactivation
	/* ---------------------------------------------- */
	public static function activate()
	{
		$errors = 0;
		$manager = new CWPlugInManager();
		// Check, create or update all database tables
		if (0 < ($e = $manager->checkTables())) {
			$errors += $e;
		}
		// Check, create or update options stored in wp_options
		if (0 < ($e = $manager->checkOptions())) {
			$errors += $e;
		}
		if (0 == $errors) {
			update_option('cw_plugin_version', CWPlugIn::VERSION);
		}
		return $manager->errorMsgs;
	}

	public static function deactivate()
	{
	}

	public function checkVersion()
	{
		$version = get_option('cw_plugin_version');
		if (empty($version) || version_compare($version, CWPlugIn::VERSION, '<')) {
			$errorMsgs = CWPlugIn::activate();
			if (0 < count($errorMsgs)) {
				$this->errorMsgs = $errorMsgs;
				add_action('all_admin_notices', [$this, 'printAdminErrors']);
			} else {
				add_action('all_admin_notices', [$this, 'printAdminUpdated']);
			}
		}
	}

	public function printAdminErrors()
	{
		if (!empty($this->errorMsgs)) {
			foreach ($this->errorMsgs as $msg) {
				echo '<div id="message" class="error">';
				echo '<p><strong>CasinoWare</strong>: ' . $msg . '</p>';
				echo '</div>' . PHP_EOL;
			}
		}
	}

	public function printAdminUpdated()
	{
		echo '<div id="message" class="updated notice is-dismissible">';
		echo '<p><strong>CasinoWare</strong>: ' . __('The databases have been updated.', CWSLUG) . '</p>';
		echo '</div>' . PHP_EOL;
	}

	/* ---------------------------------------------- */
	/* Register
	/* ---------------------------------------------- */
	/**
	 * Initialize the plugin by setting localization, filters, and administration functions.
	 */
	public function register()
	{
		// Register the stylesheet and scripts
		add_action('wp_enqueue_scripts', [$this, 'registerAssets'], 5);

		// add_action('wp_footer', [$this, 'loadScript']);

		// Register widgets
		// add_action('widgets_init', [$this, 'registerWidgets']);

		// Register ShortCodes
		add_action('init', [$this, 'registerShortCode']);

		// Blocks
		// add_action('init', [$this, 'registerBlockType']);

		// Register classes and add hooks
		if (is_admin()) {
			$classes = [CWAdmin::class];
			// Version and update check
			$this->checkVersion();
		} else {
			$classes = [CWRestApi::class];
		}
		foreach ($classes as $class) {
			$c = new $class();
			if (method_exists($c, 'register')) {
				$c->register();
			}
		}
	}

	/**
	 * Register the public-facing stylesheet and script.
	 */
	public function registerAssets()
	{
		wp_enqueue_style(CWSLUG, CWURL . 'css/cw.min.css', [], self::VERSION, 'all');
		wp_enqueue_script(CWSLUG, CWURL . 'js/cw.min.js', [], self::VERSION, 'all');
	}

	/* ---------------------------------------------- */
	/* Widget
	/* ---------------------------------------------- */
	// Register the widget
	public function registerWidgets()
	{
		// register_widget(new CWWidgetEventList($this));
	}

	/* ---------------------------------------------- */
	/* Blocks
	/* ---------------------------------------------- */
	// public function registerBlockType()
	// {
	// 	register_block_type(__DIR__);
	// }

	/* ---------------------------------------------- */
	/* Shortcode
	/* ---------------------------------------------- */
	public function registerShortCode()
	{
		add_shortcode('cw-tournament-schedule', [$this, 'renderShortCode']);
		add_shortcode('cw-tournament-list', [$this, 'renderShortCode']);
		add_shortcode('cw-tournament-results', [$this, 'renderShortCode']);
		add_shortcode('cw-tournament', [$this, 'renderShortCode']);
		add_shortcode('cw-cashgame-list', [$this, 'renderShortCode']);
		add_shortcode('cw-cashgame', [$this, 'renderShortCode']);
		add_shortcode('cw-leaderboard', [$this, 'renderShortCode']);
	}

	public function renderShortCode($attr, $content = null, $tag = '')
	{
		$obj = null;
		if (!$attr) {
			$attr = $_REQUEST;
		} else {
			$attr = array_merge($attr, $_REQUEST);
		}
		switch ($tag) {
			case 'cw-tournament-schedule':
				$obj = new CWTournamentList('schedule', $this->listID);
				$this->listID++;
				break;
			case 'cw-tournament-list':
				$obj = new CWTournamentList('list', $this->listID);
				$this->listID++;
				break;
			case 'cw-tournament-results':
				$obj = new CWTournamentList('results', $this->listID);
				$this->listID++;
				break;
			case 'cw-tournament':
				if (!($id = isset($attr['ttid']) ? intval($attr['ttid']) : null)) {
					$id = isset($attr['id']) ? intval($attr['id']) : null;
				}
				if ($id) {
					$obj = new CWTournament($id);
				}
				break;
			case 'cw-cashgame-list':
				$obj = new CWCashGameList('list', $this->listID);
				$this->listID++;
				break;
			case 'cw-cashgame':
				if (!($id = isset($attr['cgid']) ? intval($attr['cgid']) : null)) {
					$id = isset($attr['id']) ? intval($attr['id']) : null;
				}
				if ($id) {
					$obj = new CWCashGame($id);
				}
				break;
			case 'cw-leaderboard':
				$obj = new CWLeaderboard($this->listID);
				$this->listID++;
				break;
		}
		if ($obj === null) {
			$r = $this->errorBox("Unkown shortcode tag '{$tag}'");
		} else {
			$r = $obj->renderShortCode($attr);
		}
		return $r;
	}

	/* ---------------------------------------------- */
	/* Error
	/* ---------------------------------------------- */
	public function errorBox($msg, $color = 'warning')
	{
		return "<div class=\"cw-w-100 cw-text-center\"><span class=\"cw-badge cw-bg-{$color}\">{$msg}</span></div>";
	}
}
